
#include <avr/io.h>
// #include <util/delay.h>
// #include <avr/interrupt.h>
// #include <inttypes.h>
// #include <math.h>
// #include <stdint.h>
// #include <stdlib.h>

// #define F_CPU 9600000UL    // Systemtakt in Hz - Definition als unsigned long beachten >> Ohne ergeben Fehler in der Berechnung

/* Fuses: 		

Fuse High Byte
	SELFPRGEN(1) 	4 	Self Programming Enable 					1 (unprogrammed)
	DWEN(2) 		3 	debugWire Enable 							1 (unprogrammed)
	BODLEVEL1(3) 	2 	Brown-out Detector trigger level 			1 (unprogrammed)
	BODLEVEL0(3) 	1 	Brown-out Detector trigger level 			1 (unprogrammed)
	RSTDISBL(4) 	0 	External Reset disable 						1 (unprogrammed)

Fuse Low Byte
	SPIEN(1) 		7	Enable Serial Programming and Data			0 (programmed)
	EESAVE 			6	Preserve EEPROM memory through Chip Erase	1 (unprogrammed)
	WDTON(2) 		5 	Watchdog Timer always on 					1 (unprogrammed)
	CKDIV8(3) 		4 	Divide clock by 8 							0 (programmed)
	SUT1(4) 		3 	Select start-up time 						1 (unprogrammed)
	SUT0(4) 		2 	Select start-up time 						0 (programmed)
	CKSEL1(5) 		1 	Select Clock source 						1 (unprogrammed)
	CKSEL0(5) 		0 	Select Clock source 						0 (programmed)

*/

// Variablen
	uint8_t I = 53;							// Strom bei 47 Ohm in mA			Wertebereich 1..75,5
	uint8_t Duty_Cycle = 50;				// Duty_Cycle in Prozent			Wertebereich 1..100
	uint16_t U;								// aktuelle Spannung am NTC in mV	Wertebereich 1..5000
	uint8_t R;								// NTC_Widerstandswert				Wertebereich 1..128
//	int R_min;								// minimalster NTC_Widerstandswert	Wertebereich 1..50
//	int Duty_Cycle_hysterese = 5;			// Duty_Cycle Hysterese				Wertebereich 1..20
//	int Duty_Cycle_temp = 5;				// temporre Duty_Cycle				Wertebereich 1..100

// AD-Wandler-Prozedur +++ AD-Wandler-Prozedur +++ AD-Wandler-Prozedur +++ AD-Wandler-Prozedur

uint16_t ReadChannel()	
{
  uint8_t i;									// Zhler Wertebereich 1..10
  uint16_t result;								// Spannung am NTC	Wertebereich 1..1024
 
  ADMUX = 0b00000010; 								// externe Referenzspannung nutzen und Kanal PB4 auswhlen
 
  ADCSRA = (1<<ADEN) | (1<<ADPS1) | (1<<ADPS0);    	// Frequenzvorteiler 
                               						// setzen auf 8 (1) und ADC aktivieren (1)
 	
  /* nach Aktivieren des ADC wird ein "Dummy-Readout" empfohlen, man liest also einen Wert und verwirft diesen, um den ADC "warmlaufen zu lassen" */
  ADCSRA |= (1<<ADSC);              				// eine ADC-Wandlung 
  while ( ADCSRA & (1<<ADSC) ) {
     ;     											// auf Abschluss der Konvertierung warten 
  }
  result = ADCW;  									// ADCW muss einmal gelesen werden,
                  									// sonst wird Ergebnis der nchsten Wandlung
                  									// nicht bernommen.
 
  /* Messung - Mittelwert aus 5 aufeinanderfolgenden Wandlungen */
  result = 0; 
  for( i=0; i<5; i++ )
  {
    ADCSRA |= (1<<ADSC);            				// eine Wandlung "single conversion"
    while ( ADCSRA & (1<<ADSC) ) {
      ;   											// auf Abschluss der Konvertierung warten
    }
    result += ADCW;		    						// Wandlungsergebnisse aufaddieren
  }
  ADCSRA &= ~(1<<ADEN);             				// ADC deaktivieren
 
  result /= 5;		                 				// Summe durch 5 teilen = arithm. Mittelwert

  result = result * 5000/1024;						// Umwandeln in den Spannungswert
 
  return result;
}

  
// PWM-Prozedur +++ PWM-Prozedur +++ PWM-Prozedur +++ PWM-Prozedur +++ PWM-Prozedur +++

void PWM(uint8_t duty_cycle)
{
	 OCR0A = (duty_cycle*255/100);
}


     
	         
// Hauptprogramm +++ Hauptprogramm +++ Hauptprogramm +++ Hauptprogramm +++ Hauptprogramm +++ Hauptprogramm +++  

      int main (void)
  
      {
  		//	Initial I/O

		  ACSR  = 0b10000000;			// Analog-Komparator abschalten, spart Strom. ACD-Bit im ACSR auf 1. Auerdem mssen wir sicherstellen, dass die Intern Voltage Reference nicht auf den Analog Comparator geschaltet ist. Dazu muss ACBG in ACSR auf 0 gesetzt werden.
     	  DDRB  = 0b00000001;       	// PB0 ist PWM-Ausgang; Rest Eingang

		  PORTB = 0b00000001;       	// interne Pull-Ups an Port-Pins 1 bis 7 deaktiviert
		  
	//	  PORTB |= (1<<PB0); 			// setzt Bit 0 in PORTB und setzt damit Pin PB0 auf high
	//    _delay_ms(1000);           	// warten... Zeiger geht auf COLD

	//	  PORTB &= ~(1<<PB0);			// lscht Bit 0 in PORTB und setzt damit Pin PB0 auf low
	//	  _delay_ms(1000);           	// warten... Zeiger geht auf HOT

		  TCCR0A = 0b11000011;			// COM0A1 - COM0A0 (Set OC0A on Compare Match, clear OC0A at TOP) WGM01 - WGM00 (set fast PWM)
	      TCCR0B = 0b00000001;			// Prescaler = 1 => 37,647 kHz PWM-Frequenz
		  OCR0A = 0;					// clear OC0A at TOP, initialize Output Compare Register A to 0

          while(1) 						// Endlosschleife

		  {                
  			//	Analogwert U von PB4 (MUX10) einlesen mit ADC-Noise-Reduzierung

			U = ReadChannel();
            
			//	Spannungswert in NTC_Widerstandswert umwandeln  

			R = U/I;

			//  Duty_Cycle berechnen:

			Duty_Cycle = (2.8236*R - 84.199);
			
			//	Abfrage ob eine nderung der Anzeige sinnvoll ist

			//	if ((abs(Duty_Cycle_temp - Duty_Cycle)) > (Duty_Cycle_hysterese/2))
			//		{
			//				Duty_Cycle_temp = Duty_Cycle;
							PWM(Duty_Cycle);			//  PWM-Duty-Cycle an Ausgang PB0
			//		}  
 
          }
  
          		return 0;
	  }
  
 
